@@ -152,6 +152,8 @@ static const char * introspect_xml =
   "    <method name=\"GetServerPort\">\n"
   "      <arg name=\"port\" direction=\"out\" type=\"u\"/>\n"
   "    </method>\n"
+  "    <signal name=\"ServerPortChanged\">\n"
+  "    </signal>\n"
 #ifdef VINO_ENABLE_HTTP_SERVER
   "    <method name=\"GetHttpServerPort\">\n"
   "      <arg name=\"port\" direction=\"out\" type=\"u\"/>\n"
@@ -300,6 +302,34 @@ vino_dbus_listener_message_handler (DBusConnection *connection,
 #undef VINO_DBUS_INTERFACE
 }
 
+static void
+vino_dbus_listener_port_changed (VinoServer *server, VinoDBusListener *listener)
+{
+  DBusMessage *message;
+  gchar *obj_path;
+
+  dprintf (DBUS, "Emitting ServerPortChanged signal\n");
+
+  obj_path = g_strdup_printf ("/org/gnome/vino/screens/%d",
+                              gdk_screen_get_number (vino_server_get_screen (server)));
+
+  message = dbus_message_new_signal (obj_path,
+                                     "org.gnome.VinoScreen",
+                                     "ServerPortChanged");
+  g_free (obj_path);
+
+  if (!message)
+    {
+      g_warning ("Error creating signal\n");
+      return;
+    }
+
+  if (!dbus_connection_send (vino_dbus_get_connection (), message, NULL))
+    g_warning ("Error sending signal\n");
+
+  dbus_message_unref (message);
+}
+
 static DBusObjectPathVTable vino_dbus_listener_vtable =
 {
   NULL,                                  /* unregister_function */
@@ -338,6 +368,13 @@ vino_dbus_listener_set_server (VinoDBusListener *listener,
 
   dprintf (DBUS, "Object registered at path '%s'\n", obj_path);
 
+  g_signal_connect (server, "notify::alternative-port",
+		    G_CALLBACK (vino_dbus_listener_port_changed),
+		    listener);
+  g_signal_connect (server, "notify::use-alternative-port",
+		    G_CALLBACK (vino_dbus_listener_port_changed),
+		    listener);
+
   g_free (obj_path);
 }
 
@@ -388,29 +425,40 @@ vino_dbus_unref_connection (void)
   vino_dbus_connection = NULL;
 }
 
-void
+gboolean
 vino_dbus_request_name (void)
 {
 #define VINO_DBUS_BUS_NAME "org.gnome.Vino"
 
   DBusConnection *connection;
   DBusError       error;
+  int             result;
 
   if (!(connection = vino_dbus_get_connection ()))
-    return;
+    return FALSE;
 
   dbus_error_init (&error);
 
-  dbus_bus_request_name (connection, VINO_DBUS_BUS_NAME, 0, &error);
+  result = dbus_bus_request_name (connection,
+				  VINO_DBUS_BUS_NAME,
+				  DBUS_NAME_FLAG_DO_NOT_QUEUE,
+				  &error);
   if (dbus_error_is_set (&error))
     {
       g_printerr (_("Failed to acquire D-Bus name '%s'\n"),
                   error.message);
       dbus_error_free (&error);
-      return;
+      return FALSE;
+    }
+
+  if (result == DBUS_REQUEST_NAME_REPLY_EXISTS)
+    {
+      g_warning (_("Remote Desktop server already running; exiting ...\n"));
+      return FALSE;
     }
 
   dprintf (DBUS, "Successfully acquired D-Bus name '%s'\n", VINO_DBUS_BUS_NAME);
+  return TRUE;
 
 #undef VINO_DBUS_BUS_NAME
 }
