@@ -21,25 +21,36 @@
  */
 
 #include <config.h>
-
-#include "vino-prefs.h"
-
 #include <string.h>
 #include <gconf/gconf-client.h>
+#include <glib/gstdio.h>
+#include <signal.h>
+#include <fcntl.h>
+
+#include "vino-prefs.h"
 #include "vino-util.h"
+#include "vino-mdns.h"
+#include "vino-status-icon.h"
+#include "vino-background.h"
 
 #define VINO_PREFS_DIR                    "/desktop/gnome/remote_access"
 #define VINO_PREFS_ENABLED                VINO_PREFS_DIR "/enabled"
 #define VINO_PREFS_PROMPT_ENABLED         VINO_PREFS_DIR "/prompt_enabled"
 #define VINO_PREFS_VIEW_ONLY              VINO_PREFS_DIR "/view_only"
 #define VINO_PREFS_LOCAL_ONLY             VINO_PREFS_DIR "/local_only"
+#define VINO_PREFS_NETWORK_INTERFACE      VINO_PREFS_DIR "/network_interface"
 #define VINO_PREFS_USE_ALTERNATIVE_PORT   VINO_PREFS_DIR "/use_alternative_port"
 #define VINO_PREFS_ALTERNATIVE_PORT       VINO_PREFS_DIR "/alternative_port"
 #define VINO_PREFS_REQUIRE_ENCRYPTION     VINO_PREFS_DIR "/require_encryption"
 #define VINO_PREFS_AUTHENTICATION_METHODS VINO_PREFS_DIR "/authentication_methods"
 #define VINO_PREFS_VNC_PASSWORD           VINO_PREFS_DIR "/vnc_password"
+#define VINO_PREFS_LOCK_SCREEN            VINO_PREFS_DIR "/lock_screen_on_disconnect"
+#define VINO_PREFS_ICON_VISIBILITY        VINO_PREFS_DIR "/icon_visibility"
+#define VINO_PREFS_DISABLE_BACKGROUND     VINO_PREFS_DIR "/disable_background"
 
-#define VINO_N_LISTENERS 9
+#define VINO_N_LISTENERS                  12
+
+#define VINO_PREFS_LOCKFILE               "vino-server.lock"
 
 static GConfClient *vino_client  = NULL;
 static GSList      *vino_servers = NULL;
@@ -48,13 +59,15 @@ static guint        vino_listeners [VINO_N_LISTENERS] = { 0, };
 static gboolean        vino_enabled              = FALSE;
 static gboolean        vino_prompt_enabled       = FALSE;
 static gboolean        vino_view_only            = FALSE;
-static gboolean        vino_local_only           = FALSE;
+static char           *vino_network_interface    = NULL;
 static gboolean        vino_require_encryption   = FALSE;
 static VinoAuthMethod  vino_auth_methods         = VINO_AUTH_VNC;
 static char           *vino_vnc_password         = NULL;
 static gboolean        vino_use_alternative_port = FALSE;
 static int             vino_alternative_port     = VINO_SERVER_DEFAULT_PORT;
-
+static gboolean        vino_lock_screen          = FALSE;
+static gboolean        vino_disable_background   = FALSE;
+static VinoStatusIconVisibility vino_icon_visibility = VINO_STATUS_ICON_VISIBILITY_CLIENT;
 
 static void
 vino_prefs_enabled_changed (GConfClient *client,
@@ -76,6 +89,11 @@ vino_prefs_enabled_changed (GConfClient *client,
   
   dprintf (PREFS, "Access enabled changed: %s\n", vino_enabled ? "(true)" : "(false)");
 
+  if (vino_enabled)
+    vino_mdns_start ();
+  else
+    vino_mdns_stop ();
+
   for (l = vino_servers; l; l = l->next)
     vino_server_set_on_hold (l->data, !enabled);
 }
@@ -129,27 +147,35 @@ vino_prefs_view_only_changed (GConfClient *client,
 }
 
 static void
-vino_prefs_local_only_changed (GConfClient *client,
-			       guint	    cnxn_id,
-			       GConfEntry  *entry)
+vino_prefs_network_interface_changed (GConfClient *client,
+                                      guint        cnxn_id,
+                                      GConfEntry  *entry)
 {
-  gboolean  local_only;
-  GSList   *l;
+  const char *network_interface;
+  GSList     *l;
 
-  if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
+  if (!entry->value || entry->value->type != GCONF_VALUE_STRING)
     return;
 
-  local_only = gconf_value_get_bool (entry->value) != FALSE;
+  network_interface = gconf_value_get_string (entry->value);
 
-  if (vino_local_only == local_only)
+  if (!network_interface && !vino_network_interface)
     return;
 
-  vino_local_only = local_only;
+  if (network_interface && vino_network_interface &&
+      !g_strcasecmp (network_interface, vino_network_interface))
+    return;
 
-  dprintf (PREFS, "Local only changed: %s\n", vino_local_only ? "(true)" : "(false)");
+  if (vino_network_interface)
+    g_free (vino_network_interface);
+
+  vino_network_interface = g_strdup (network_interface);
+
+  dprintf (PREFS, "Network Interface for bind() changed: %s\n",
+       vino_network_interface ? vino_network_interface : "(null)");
 
   for (l = vino_servers; l; l = l->next)
-    vino_server_set_local_only (l->data, local_only);
+    vino_server_set_network_interface (l->data, network_interface);
 }
 
 static void
@@ -263,6 +289,14 @@ vino_prefs_vnc_password_changed (GConfClient *client,
     vino_server_set_vnc_password (l->data, vnc_password);
 }
 
+static void
+vino_prefs_restart_mdns (VinoServer *server, gpointer data)
+{
+  vino_mdns_stop ();
+  vino_mdns_add_service ("_rfb._tcp", vino_server_get_port (server));
+  vino_mdns_start ();
+}
+
 static void
 vino_prefs_use_alternative_port_changed (GConfClient *client,
                                          guint        cnxn_id,
@@ -312,15 +346,111 @@ vino_prefs_alternative_port_changed (GConfClient *client,
     vino_server_set_alternative_port (l->data, vino_alternative_port);
 }
 
+static void
+vino_prefs_lock_screen_changed (GConfClient *client,
+			        guint	    cnxn_id,
+			        GConfEntry  *entry)
+{
+  gboolean  lock_screen;
+  GSList   *l;
+
+  if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
+    return;
+
+  lock_screen = gconf_value_get_bool (entry->value) != FALSE;
+
+  if (vino_lock_screen == lock_screen)
+    return;
+
+  vino_lock_screen = lock_screen;
+
+  dprintf (PREFS, "Lock Screen changed: %s\n", vino_lock_screen ? "(true)" : "(false)");
+
+  for (l = vino_servers; l; l = l->next)
+    vino_server_set_lock_screen (l->data, lock_screen);
+}
+
+static VinoStatusIconVisibility
+vino_prefs_icon_visibility_from_string (const char *value)
+{
+  VinoStatusIconVisibility ret_value = VINO_STATUS_ICON_VISIBILITY_INVALID;
+
+  if (!g_ascii_strcasecmp (value, "always"))
+    ret_value = VINO_STATUS_ICON_VISIBILITY_ALWAYS;
+  else if (!g_ascii_strcasecmp (value, "client"))
+    ret_value = VINO_STATUS_ICON_VISIBILITY_CLIENT;
+  else if (!g_ascii_strcasecmp (value, "never"))
+    ret_value = VINO_STATUS_ICON_VISIBILITY_NEVER;
+
+  return ret_value;
+}
+
+static void
+vino_prefs_icon_visibility_changed (GConfClient *client,
+				    guint	 cnxn_id,
+				    GConfEntry  *entry)
+{
+  const gchar  *entry_str;
+  GSList *l;
+  VinoStatusIconVisibility visibility;
+
+  if (!entry->value || entry->value->type != GCONF_VALUE_STRING)
+    return;
+
+  entry_str = gconf_value_get_string (entry->value);
+  visibility = vino_prefs_icon_visibility_from_string (entry_str);
+
+  if (visibility == vino_icon_visibility)
+    return;
+
+  vino_icon_visibility = visibility;
+
+  dprintf (PREFS, "Icon visibility changed: %s\n", entry_str);
+
+  for (l = vino_servers; l; l = l->next)
+    {
+      VinoStatusIcon *icon;
+
+      icon = vino_server_get_status_icon (l->data);
+      vino_status_icon_set_visibility (icon, visibility);
+    }
+}
+
+static void
+vino_prefs_disable_background_changed (GConfClient *client,
+                                       guint       cnxn_id,
+                                       GConfEntry  *entry)
+{
+  gboolean  disable_background;
+  GSList   *l;
+
+  if (!entry->value || entry->value->type != GCONF_VALUE_BOOL)
+    return;
+
+  disable_background = gconf_value_get_bool (entry->value) != FALSE;
+
+  if (vino_disable_background == disable_background)
+    return;
+
+  vino_disable_background = disable_background;
+
+  dprintf (PREFS, "Disable background changed: %s\n", vino_disable_background ? "(true)" : "(false)");
+
+  for (l = vino_servers; l; l = l->next)
+    vino_server_set_disable_background (l->data, disable_background);
+}
+
 void
 vino_prefs_create_server (GdkScreen *screen)
 {
-  VinoServer *server;
+  VinoServer     *server;
+  VinoStatusIcon *icon;
+
 
   server = g_object_new (VINO_TYPE_SERVER,
 			 "prompt-enabled",       vino_prompt_enabled,
 			 "view-only",            vino_view_only,
-			 "local-only",           vino_local_only,
+			 "network-interface",    vino_network_interface,
 			 "use-alternative-port", vino_use_alternative_port,
 			 "alternative-port",     vino_alternative_port,
 			 "auth-methods",         vino_auth_methods,
@@ -328,9 +458,91 @@ vino_prefs_create_server (GdkScreen *screen)
 			 "vnc-password",         vino_vnc_password,
 			 "on-hold",              !vino_enabled,
 			 "screen",               screen,
+			 "lock-screen",          vino_lock_screen,
+			 "disable-background",   vino_disable_background,
 			 NULL);
 
   vino_servers = g_slist_prepend (vino_servers, server);
+  if (vino_enabled)
+    vino_mdns_start ();
+
+  g_signal_connect (server, "notify::alternative-port", G_CALLBACK (vino_prefs_restart_mdns), NULL);
+  g_signal_connect (server, "notify::use-alternative-port", G_CALLBACK(vino_prefs_restart_mdns), NULL);
+
+  icon = vino_server_get_status_icon (server);
+  vino_status_icon_set_visibility (icon, vino_icon_visibility);
+}
+
+static void
+vino_prefs_restore (void)
+{
+  if (vino_background_get_status ())
+    vino_background_draw (TRUE);
+}
+
+static gchar *
+vino_prefs_lock_filename (void)
+{
+  gchar *dir;
+
+  dir = g_build_filename (g_get_user_data_dir (),
+			 "vino",
+			  NULL);
+  if (!g_file_test (dir, G_FILE_TEST_EXISTS))
+    g_mkdir_with_parents (dir, 0755);
+
+  g_free (dir);
+
+  return g_build_filename (g_get_user_data_dir (),
+			   "vino",
+			    VINO_PREFS_LOCKFILE,
+			    NULL);
+}
+
+static gboolean
+vino_prefs_lock (void)
+{
+  gchar    *lockfile;
+  gboolean  res;
+
+  res = FALSE;
+  lockfile = vino_prefs_lock_filename ();
+
+  if (g_file_test (lockfile, G_FILE_TEST_EXISTS))
+    {
+      dprintf (PREFS, "WARNING: The lock file (%s) already exists\n", lockfile);
+    }
+  else
+    {
+      g_creat (lockfile, 0644);
+      res = TRUE;
+    }
+
+  g_free (lockfile);
+  return res;
+}
+
+static gboolean
+vino_prefs_unlock (void)
+{
+  gchar    *lockfile;
+  gboolean  res;
+
+  res = FALSE;
+  lockfile = vino_prefs_lock_filename ();
+
+  if (!g_file_test (lockfile, G_FILE_TEST_EXISTS))
+    {
+      dprintf (PREFS, "WARNING: Lock file (%s) not found!\n", lockfile);
+    }
+  else
+    {
+      g_unlink (lockfile);
+      res = TRUE;
+    }
+
+  g_free (lockfile);
+  return res;
 }
 
 void
@@ -338,6 +550,11 @@ vino_prefs_init (gboolean view_only)
 {
   GSList *auth_methods_list, *l;
   int i = 0;
+  char *key_str;
+  
+  signal (SIGQUIT, vino_background_handler); /* Ctrl+C */
+  signal (SIGTERM, vino_background_handler); /* kill -15 */
+  signal (SIGSEGV, vino_background_handler); /* Segmentation fault */
 
   vino_client = gconf_client_get_default ();
 
@@ -346,6 +563,9 @@ vino_prefs_init (gboolean view_only)
 			GCONF_CLIENT_PRELOAD_ONELEVEL,
 			NULL);
 
+  if(!vino_prefs_lock ())
+    vino_prefs_restore ();
+
   vino_enabled = gconf_client_get_bool (vino_client, VINO_PREFS_ENABLED, NULL);
   dprintf (PREFS, "Access enabled: %s\n", vino_enabled ? "(true)" : "(false)");
 
@@ -365,15 +585,23 @@ vino_prefs_init (gboolean view_only)
 					      NULL);
     }
   dprintf (PREFS, "View only: %s\n", vino_view_only ? "(true)" : "(false)");
-      
-  vino_local_only = gconf_client_get_bool (vino_client,
-					   VINO_PREFS_LOCAL_ONLY,
-					   NULL);
-  dprintf (PREFS, "Local only: %s\n", vino_local_only ? "(true)" : "(false)");
+ 
+  vino_network_interface = gconf_client_get_string (vino_client,
+						    VINO_PREFS_NETWORK_INTERFACE,
+						    NULL);
+  /* Check for old key, local_only, vino <= 2.24 */
+  if (!vino_network_interface && gconf_client_get_bool (vino_client, VINO_PREFS_LOCAL_ONLY, NULL))
+    {
+      gconf_client_set_string (vino_client, VINO_PREFS_NETWORK_INTERFACE, "lo", NULL);
+      vino_network_interface = g_strdup ("lo");
+    }
+  dprintf (PREFS, "Network interface: %s\n", 
+                vino_network_interface ? vino_network_interface : "all");
 
   vino_use_alternative_port = gconf_client_get_bool (vino_client,
                                                      VINO_PREFS_USE_ALTERNATIVE_PORT,
                                                      NULL);
+    
   dprintf (PREFS, "Use alternative port: %s\n",
            vino_use_alternative_port ? "(true)" : "(false)");
 
@@ -407,6 +635,24 @@ vino_prefs_init (gboolean view_only)
 					       NULL);
   dprintf (PREFS, "Encoded password: %s\n", vino_vnc_password ? vino_vnc_password : "(null)");
 
+  vino_lock_screen = gconf_client_get_bool (vino_client,
+                                            VINO_PREFS_LOCK_SCREEN,
+                                            NULL);
+  dprintf (PREFS, "Lock screen on disconnect: %s\n",
+           vino_lock_screen ? "(true)" : "(false)");
+
+  vino_disable_background = gconf_client_get_bool (vino_client,
+                                                   VINO_PREFS_DISABLE_BACKGROUND,
+                                                   NULL);
+  dprintf (PREFS, "Disable background: %s\n", vino_disable_background ? "(true)" : "(false)");
+
+  key_str = gconf_client_get_string (vino_client,
+                                     VINO_PREFS_ICON_VISIBILITY,
+                                     NULL);
+  vino_icon_visibility = vino_prefs_icon_visibility_from_string (key_str);
+  dprintf (PREFS, "Icon policy: %s\n", key_str);
+  g_free (key_str);
+
   vino_listeners [i] =
     gconf_client_notify_add (vino_client,
 			     VINO_PREFS_ENABLED,
@@ -433,8 +679,8 @@ vino_prefs_init (gboolean view_only)
 
   vino_listeners [i] =
     gconf_client_notify_add (vino_client,
-			     VINO_PREFS_LOCAL_ONLY,
-			     (GConfClientNotifyFunc) vino_prefs_local_only_changed,
+			     VINO_PREFS_NETWORK_INTERFACE,
+			     (GConfClientNotifyFunc) vino_prefs_network_interface_changed,
 			     NULL, NULL, NULL);
   i++;
 
@@ -474,6 +720,30 @@ vino_prefs_init (gboolean view_only)
 			     NULL, NULL, NULL);
   i++;
 
+  vino_listeners [i] =
+    gconf_client_notify_add (vino_client,
+			     VINO_PREFS_LOCK_SCREEN,
+			     (GConfClientNotifyFunc) vino_prefs_lock_screen_changed,
+			     NULL, NULL, NULL);
+
+  i++;
+
+  vino_listeners [i] =
+    gconf_client_notify_add (vino_client,
+			     VINO_PREFS_ICON_VISIBILITY,
+			     (GConfClientNotifyFunc) vino_prefs_icon_visibility_changed,
+			     NULL, NULL, NULL);
+
+  i++;
+
+  vino_listeners [i] =
+    gconf_client_notify_add (vino_client,
+			     VINO_PREFS_DISABLE_BACKGROUND,
+			     (GConfClientNotifyFunc) vino_prefs_disable_background_changed,
+			     NULL, NULL, NULL);
+
+  i++;
+
   g_assert (i == VINO_N_LISTENERS);
 }
 
@@ -491,7 +761,11 @@ vino_prefs_shutdown (void)
   if (vino_vnc_password)
     g_free (vino_vnc_password);
   vino_vnc_password = NULL;
-      
+
+  if (vino_network_interface)
+    g_free (vino_network_interface);
+  vino_network_interface = NULL;
+
   for (i = 0; i < VINO_N_LISTENERS; i++) {
     if (vino_listeners [i])
       gconf_client_notify_remove (vino_client, vino_listeners [i]);
@@ -500,4 +774,6 @@ vino_prefs_shutdown (void)
 
   g_object_unref (vino_client);
   vino_client = NULL;
+
+  vino_prefs_unlock ();
 }
